package pobyty.business;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.smp.SmpException;
import pobyty.integration.DerbyDAOFactory;
import pobyty.model.Datum;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.PobytyIntegrityException;
import pobyty.model.Poznamka;
import pobyty.model.PrihlaskaKlient;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.doprovod.Doprovod;
import pobyty.model.doprovod.DoprovodId;
import pobyty.model.klient.Indikace;
import pobyty.model.klient.Klient;
import pobyty.model.klient.KlientId;
import pobyty.model.klient.Kontraindikace;
import pobyty.model.prihlaska.Prihlaska;
import pobyty.model.prihlaska.StavPrihlasky;
import pobyty.model.skupina.Skupina;
import pobyty.model.skupina.SkupinaId;
import pobyty.model.skupina.VekKlientu;
import pobyty.model.turnus.Cena;
import pobyty.model.turnus.Kapacita;
import pobyty.model.turnus.Lokalita;
import pobyty.model.turnus.Turnus;
import pobyty.model.turnus.TurnusId;
import pobyty.model.turnus.Zkratka;
import pobyty.model.uzivatel.Heslo;
import pobyty.model.uzivatel.RoleUzivatele;
import pobyty.model.uzivatel.Username;
import pobyty.model.uzivatel.Uzivatel;

public class FacadeImpl extends Facade {

    UzivatelDAO uzivatelDAO;
    TurnusDAO turnusyDAO;
    SkupinaDAO skupinaDAO;
    PrihlaskaDAO prihlaskaDAO;
    KlientDAO klientDAO;
    DoprovodDAO doprovodDAO;
    PobytyDAOFactory factory = new DerbyDAOFactory();

    public FacadeImpl() {
        try {
            uzivatelDAO = factory.createUzivatelDAO();
            turnusyDAO = factory.createTurnusDAO();
            skupinaDAO = factory.createSkupinaDAO();
            klientDAO = factory.createKlientDAO();
            prihlaskaDAO = factory.createPrihlaskaDAO();
            doprovodDAO = factory.createDoprovodDAO();

        } catch (SQLException ex) {
            logger.log(Level.SEVERE, null, ex);
        }
    }
    private static final Logger logger = Logger.getLogger(Facade.class.getName());

    /**
     * ************************************************ Klient
     */
    @Override
    public void addKlient(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno,
            Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon,
            Indikace indikace, Kontraindikace kontraindikace) throws SmpException {
        klientDAO.add(klientId, prijmeni, jmeno, narozeni, mejl, ulice, psc, mesto, telefon, indikace, kontraindikace);
    }

    @Override
    public Klient findKlient(KlientId id) throws SmpException {
        return klientDAO.find(id);
    }

    @Override
    public List<Klient> klienti() throws SmpException {
        return klientDAO.all();
    }

    @Override
    public List<Klient> klienti(String filtrPrijmeni, String filtrJmeno) throws SmpException {
        List<Klient> klienti = new ArrayList<>();
        for (Klient klient : klienti()) {
            if (klient.matchFilter(filtrPrijmeni, filtrJmeno)) {
                klienti.add(klient);
            }
        }
        return klienti;
    }

    @Override
    public void updateKlient(KlientId klientId, Mejl mejl, Ulice ulice, Psc psc,
            Mesto mesto, Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException {
        klientDAO.update(klientId, mejl, ulice, psc, mesto, telefon, indikace, kontraindikace);
    }

    @Override
    public void updateKlient(KlientId klientId, Prijmeni prijmeni, Jmeno jmeno, Datum narozeni, Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon, Indikace indikace, Kontraindikace kontraindikace) throws SmpException {
        klientDAO.update(klientId, prijmeni, jmeno, narozeni, mejl, ulice, psc, mesto, telefon, indikace, kontraindikace);
    }

    @Override
    public void deleteKlient(KlientId klientId) throws SmpException, PobytyIntegrityException {
        klientDAO.delete(klientId);
    }

    /**
     * ************************************************ Uzivatel
     */
    @Override
    public void addUzivatel(Username username, Prijmeni prijmeni, Jmeno jmeno,
            RoleUzivatele roleUzivatele, Heslo heslo) throws SmpException {
        uzivatelDAO.add(username, prijmeni, jmeno, roleUzivatele, heslo);
    }

    @Override
    public Uzivatel findUzivatel(Username username) throws SmpException {
        return uzivatelDAO.find(username);
    }

    @Override
    public List<Uzivatel> allUzivatel() throws SmpException {
        return uzivatelDAO.all();
    }

    @Override
    public void deleteUzivatel(Username username) throws SmpException {
        uzivatelDAO.delete(username);
    }

    @Override
    public void zmenitHeslo(Username username, Heslo noveHeslo) throws SmpException {
        uzivatelDAO.update(username, noveHeslo);
    }

    /**
     * ************************************************ Skupina
     */
    @Override
    public void addSkupina(TurnusId turnusId, VekKlientu vekKlientu, DoprovodId doprovodId) throws SmpException {
        skupinaDAO.add(turnusId, vekKlientu, doprovodId);
    }

    @Override
    public Skupina findSkupina(SkupinaId skupinaId) throws SmpException {
        return skupinaDAO.find(skupinaId);
    }

    @Override
    public List<Skupina> getSkupinyTurnusu(TurnusId turnusId) throws SmpException {
        return skupinaDAO.all(turnusId);
    }

    @Override
    public void updateSkupina(SkupinaId skupinaId, DoprovodId doprovodId) throws SmpException {
        skupinaDAO.update(skupinaId, doprovodId);
    }

    @Override
    public void deleteSkupina(SkupinaId skupinaId) throws SmpException, PobytyIntegrityException {
        skupinaDAO.delete(skupinaId);
    }

    @Override
    public List<Klient> kandidatiSkupiny(Skupina skupina) throws SmpException {
        int vek = skupina.getVekKlientu().getVek();
        List<Klient> kandidatiSkupiny = new ArrayList<>();
        for (Prihlaska prihlaska : prihlasky()) {
            if (prihlaska.getSkupinaId().equals(SkupinaId.NEDEF)
                    && prihlaska.getStavPrihlasky() == StavPrihlasky.ZARAZEN
                    && prihlaska.getTurnus().getTurnusId().equals(skupina.getTurnus().getTurnusId())) {
                Klient klient = findKlient(prihlaska.getKlientId());
                kandidatiSkupiny.add(klient);
            }
        }
        return kandidatiSkupiny;
    }

    /**
     * ************************************************ Prihlaska
     */
    @Override
    public List<PrihlaskaKlient> klientiSkupiny(SkupinaId skupinaId) throws SmpException {
        return prihlaskaDAO.prihlaskaKlientJoin(skupinaId);
    }

    @Override
    public void addPrihlaska(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException {
        prihlaskaDAO.add(klientId, turnusId, poznamka);
    }

    @Override
    public Prihlaska findPrihlaska(KlientId klientId) throws SmpException {
        return prihlaskaDAO.find(klientId);
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(String filtrPrijmeni, String filtrJmeno) throws SmpException {

        List<PrihlaskaKlient> klienti = new ArrayList<>();
        for (PrihlaskaKlient klient : prihlaskaDAO.prihlaskaKlientJoin()) {
            if (klient.matchFilter(filtrPrijmeni, filtrJmeno)) {
                klienti.add(klient);
            }
        }
        return klienti;
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin() throws SmpException {
        return prihlaskaDAO.prihlaskaKlientJoin();
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(TurnusId turnusId) throws SmpException {
        return prihlaskaDAO.prihlaskaKlientJoin(turnusId);
    }

    @Override
    public List<PrihlaskaKlient> prihlaskaKlientJoin(StavPrihlasky stavPrihlasky) throws SmpException {
        return prihlaskaDAO.prihlaskaKlientJoin(stavPrihlasky);
    }

    @Override
    public List<Prihlaska> prihlasky() throws SmpException {
        return prihlaskaDAO.all();
    }

    @Override
    public void updatePrihlaska(KlientId klientId, StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException {
        prihlaskaDAO.update(klientId, stavPrihlasky, poznamka);
    }

    @Override
    public void updatePrihlaska(KlientId klientId, TurnusId turnusId, Poznamka poznamka) throws SmpException {
        prihlaskaDAO.update(klientId, turnusId, poznamka);
    }

    @Override
    public void updatePrihlaska(KlientId klientId, TurnusId turnusId,
            StavPrihlasky stavPrihlasky, Poznamka poznamka) throws SmpException {
        prihlaskaDAO.update(klientId, turnusId, stavPrihlasky, poznamka);
    }

    @Override
    public void updatePrihlaska(KlientId klientId, SkupinaId skupinaId) throws SmpException {
        prihlaskaDAO.update(klientId, skupinaId);
    }

    @Override
    public void deletePrihlaska(KlientId klientId) throws SmpException {
        prihlaskaDAO.delete(klientId);
    }

    @Override
    public int pocetKlientuSkupiny(SkupinaId skupinaId) throws SmpException {
        return prihlaskaDAO.clientsCount(skupinaId);
    }

    @Override
    public int pocetPrihlasenychNaTurnus(TurnusId turnusId) throws SmpException {
        return prihlaskaDAO.clientsCount(turnusId);
    }

    /**
     * ************************************************ Turnus
     */
    @Override
    public void addTurnus(Datum zacatek, Datum konec, Lokalita lokalita,
            Cena cena, Kapacita kapacita, Zkratka zkratka) throws SmpException {
        turnusyDAO.add(zacatek, konec, lokalita, cena, kapacita, zkratka);
    }

    @Override
    public Turnus findTurnus(TurnusId turnusId) throws SmpException {
        return turnusyDAO.find(turnusId);
    }

    @Override
    public List<Turnus> turnusy() throws SmpException {
        return turnusyDAO.all();
    }

    @Override
    public void updateTurnus(TurnusId turnusId, Zkratka zkratka, Datum zacatek, Datum konec,
            Lokalita lokalita, Cena cena, Kapacita kapacita) throws SmpException {
        turnusyDAO.update(turnusId, zkratka, zacatek, konec, lokalita, cena, kapacita);
    }

    @Override
    public void deleteTurnus(TurnusId turnusId) throws SmpException, PobytyIntegrityException {
        turnusyDAO.delete(turnusId);
    }

    /**
     * ************************************************ Doprovod
     */
    @Override
    public void addDoprovod(Prijmeni prijmeni, Jmeno jmeno, Datum narozeni,
            Mejl mejl, Ulice ulice, Psc psc, Mesto mesto, Telefon telefon) throws SmpException {
        doprovodDAO.add(prijmeni, jmeno, narozeni, mejl, ulice, mesto, psc, telefon);
    }

    @Override
    public Doprovod findDoprovod(DoprovodId doprovodId) throws SmpException {
        return doprovodDAO.find(doprovodId);
    }

    @Override
    public List<Doprovod> doprovody() throws SmpException {
        return doprovodDAO.all();
    }

    @Override
    public List<Doprovod> doprovody(String filtrPrijmeni, String jmeno) throws SmpException {
        List<Doprovod> doprovody = new ArrayList<>();
        for (Doprovod doprovod : doprovody()) {
            if (doprovod.matchFilter(filtrPrijmeni, jmeno)) {
                doprovody.add(doprovod);
            }
        }
        return doprovody;
    }

    @Override
    public void updateDoprovod(DoprovodId doprovodId, Prijmeni prijmeni, Jmeno jmeno, Mejl mejl, Ulice ulice,
            Mesto mesto, Psc psc, Telefon telefon) throws SmpException {
        doprovodDAO.update(doprovodId, prijmeni, jmeno, mejl, ulice, mesto, psc, telefon);
    }

    @Override
    public void deleteDoprovod(DoprovodId doprovodId) throws SmpException {
        doprovodDAO.delete(doprovodId);
    }
}
