package pobyty.integration;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import pobyty.Parametry;
import pobyty.business.DoprovodDAO;
import pobyty.business.DoprovodyInit;
import pobyty.business.KlientDAO;
import pobyty.business.KlientiInit;
import pobyty.business.PobytyDAOFactory;
import pobyty.business.PrihlaskaDAO;
import pobyty.business.SkupinaDAO;
import pobyty.business.TurnusDAO;
import pobyty.business.UzivatelDAO;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.OsobaAdresa;
import pobyty.model.Poznamka;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.doprovod.DoprovodId;
import pobyty.model.klient.Indikace;
import pobyty.model.klient.KlientId;
import pobyty.model.klient.Kontraindikace;
import pobyty.model.prihlaska.StavPrihlasky;
import pobyty.model.skupina.SkupinaId;
import pobyty.model.skupina.VekKlientu;
import pobyty.model.turnus.Cena;
import pobyty.model.turnus.Kapacita;
import pobyty.model.turnus.Lokalita;
import pobyty.model.turnus.Turnus;
import pobyty.model.turnus.TurnusId;
import pobyty.model.turnus.Zkratka;
import pobyty.model.uzivatel.Heslo;
import pobyty.model.uzivatel.RoleUzivatele;
import pobyty.model.uzivatel.Username;

public class DerbyDAOFactory implements PobytyDAOFactory {

    Connection conn;
    static final Logger logger = Logger.getLogger(DerbyDAOFactory.class.getName());

    public DerbyDAOFactory() {
        try {
            try {                
                InitialContext ic = new InitialContext();
                DataSource ds = (DataSource) ic.lookup(Parametry.getInstance().getParameter("JNDIName"));
                conn = ds.getConnection();
            } catch (NamingException ex) {
                Logger.getLogger(DerbyDAOFactory.class.getName()).log(Level.SEVERE, null, ex);
            }

            Statement s = conn.createStatement();
            DatabaseMetaData md = conn.getMetaData();
            String ct;
            ResultSet rs = md.getTables(null, null, "KLIENT", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table KLIENT generated");
                ct = "CREATE TABLE KLIENT ("
                        + String.format("%s  VARCHAR(%s) NOT NULL,", KlientId.colName(), KlientId.KLIENTID_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Prijmeni.colName(), Prijmeni.PRIJMENI_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Jmeno.colName(), Jmeno.JMENO_SIZE)
                        + String.format("%s  DATE NOT NULL ,", OsobaAdresa.COLUMN_NAROZENI)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Mejl.colName(), Mejl.MEJL_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Ulice.colName(), Ulice.ULICE_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Mesto.colName(), Mesto.MESTO_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Psc.colName(), Psc.PSC_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Telefon.colName(), Telefon.TELEFON_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Indikace.colName(), Indikace.size())
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Kontraindikace.colName(), Kontraindikace.size())
                        + String.format("PRIMARY KEY (%s))", KlientId.colName());
                s.executeUpdate(ct);
                KlientiInit.init();
            }
            rs = md.getTables(null, null, "TURNUS", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table TURNUS generated");
                ct = "CREATE TABLE TURNUS ("
                        + String.format("%s  INT NOT NULL GENERATED ALWAYS AS IDENTITY,", TurnusId.colName())
                        + String.format("%s  DATE NOT NULL,", Turnus.COLUMN_ZACATEK)
                        + String.format("%s  DATE NOT NULL,", Turnus.COLUMN_KONEC)
                        + String.format("%s  VARCHAR(%s)NOT NULL,", Lokalita.colName(), Lokalita.LOKALITA_SIZE)
                        + String.format("%s  INT NOT NULL,", Cena.colName())
                        + String.format("%s  INT NOT NULL,", Kapacita.colName())
                        + String.format("%s  VARCHAR(%s) NOT NULL ,", Zkratka.colName(), Zkratka.ZKRATKA_SIZE)
                        + String.format("PRIMARY KEY (%s))", TurnusId.colName());
                s.executeUpdate(ct);
            }
            rs = md.getTables(null, null, "DOPROVOD", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table DOPROVOD generated");
                ct = "CREATE TABLE DOPROVOD ("
                        + String.format("%s  INT NOT NULL GENERATED ALWAYS AS IDENTITY,", DoprovodId.colName())
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Prijmeni.colName(), Prijmeni.PRIJMENI_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Jmeno.colName(), Jmeno.JMENO_SIZE)
                        + String.format("%s  DATE NOT NULL,", OsobaAdresa.COLUMN_NAROZENI)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Mejl.colName(), Mejl.MEJL_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Ulice.colName(), Ulice.ULICE_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Mesto.colName(), Mesto.MESTO_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Psc.colName(), Psc.PSC_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Telefon.colName(), Telefon.TELEFON_SIZE)
                        + String.format("PRIMARY KEY (%s))", DoprovodId.colName());
                s.executeUpdate(ct);
                DoprovodyInit.init();
            }
            rs = md.getTables(null, null, "SKUPINA", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table SKUPINA generated");
                ct = "CREATE TABLE SKUPINA ("
                        + String.format("%s  INT NOT NULL GENERATED ALWAYS AS IDENTITY,", SkupinaId.colName())
                        + String.format("%s  INT NOT NULL,", TurnusId.colName())
                        + String.format("%s  INT NOT NULL,", VekKlientu.colName())
                        + String.format("%s  INT,", DoprovodId.colName())
                        + String.format("PRIMARY KEY (%s),", SkupinaId.colName())
                        + String.format("FOREIGN KEY (%s) REFERENCES TURNUS(%S),", TurnusId.colName(), TurnusId.colName())
                        + String.format("FOREIGN KEY (%s) REFERENCES DOPROVOD(%S)) ", DoprovodId.colName(), DoprovodId.colName());
                s.executeUpdate(ct);
            }
            rs = md.getTables(null, null, "PRIHLASKA", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table PRIHLASKA generated");
                ct = "CREATE TABLE PRIHLASKA ("
                        + String.format("%s  VARCHAR(%s) NOT NULL,", KlientId.colName(), KlientId.KLIENTID_SIZE)
                        + String.format("%s  INT NOT NULL,", TurnusId.colName())
                        + String.format("%s  INT,", SkupinaId.colName())
                        + String.format("%s  VARCHAR(%s) NOT NULL,", StavPrihlasky.colName(), StavPrihlasky.size())
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Poznamka.colName(), Poznamka.POZNAMKA_SIZE)
                        + String.format("PRIMARY KEY (%s),", KlientId.colName())
                        + String.format("FOREIGN KEY (%s) REFERENCES KLIENT(%S), ", KlientId.colName(), KlientId.colName())
                        + String.format("FOREIGN KEY (%s) REFERENCES TURNUS(%S), ", TurnusId.colName(), TurnusId.colName())
                        + String.format("FOREIGN KEY (%s) REFERENCES SKUPINA(%S)) ", SkupinaId.colName(), SkupinaId.colName());
                System.out.println(ct);
                s.executeUpdate(ct);

            }
            rs = md.getTables(null, null, "UZIVATEL", null);
            if (!rs.next()) {
                logger.log(Level.INFO, "Table UZIVATEL generated");
                ct = "CREATE TABLE UZIVATEL ("
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Username.colName(), Username.USERNAME_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Prijmeni.colName(), Prijmeni.PRIJMENI_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Jmeno.colName(), Jmeno.JMENO_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", RoleUzivatele.colName(), RoleUzivatele.ROLEUZIVATELE_SIZE)
                        + String.format("%s  VARCHAR(%s) NOT NULL,", Heslo.colName(), Heslo.MD5_SIZE)
                        + String.format("PRIMARY KEY (%s))", Username.colName());
                s.executeUpdate(ct);
            }
        } catch (SQLException ex) {
            logger.log(Level.SEVERE, "", ex);
        }
    }

    @Override
    public UzivatelDAO createUzivatelDAO() throws SQLException {
        return new UzivatelDAODerby(conn);
    }

    @Override
    public KlientDAO createKlientDAO() throws SQLException {
        return new KlientDAODerby(conn);
    }

    @Override
    public PrihlaskaDAO createPrihlaskaDAO() throws SQLException {
        return new PrihlaskaDAODerby(conn);
    }

    @Override
    public TurnusDAO createTurnusDAO() throws SQLException {
        return new TurnusDAODerby(conn);
    }

    @Override
    public SkupinaDAO createSkupinaDAO() throws SQLException {
        return new SkupinaDAODerby(conn);
    }

    @Override
    public DoprovodDAO createDoprovodDAO() throws SQLException {
        return new DoprovodDAODerby(conn);
    }
}
