package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import org.smp.SmpException;
import pobyty.business.DoprovodDAO;
import pobyty.model.Datum;
import pobyty.model.Jmeno;
import pobyty.model.Mejl;
import pobyty.model.Mesto;
import pobyty.model.OsobaAdresa;
import pobyty.model.Prijmeni;
import pobyty.model.Psc;
import pobyty.model.Telefon;
import pobyty.model.Ulice;
import pobyty.model.doprovod.Doprovod;
import pobyty.model.doprovod.DoprovodId;

public class DoprovodDAODerby implements DoprovodDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updatePs;

    public DoprovodDAODerby(Connection connection) throws SQLException {
        createPs = connection.prepareStatement("INSERT INTO DOPROVOD VALUES(DEFAULT, ?, ?, ?, ?, ?, ?, ?, ?)");
        findPs = connection.prepareStatement(
                String.format("SELECT * FROM DOPROVOD WHERE %s = ?", DoprovodId.colName()));
        findAllPs = connection.prepareStatement("SELECT * FROM DOPROVOD");
        updatePs = connection.prepareStatement(
                String.format("UPDATE DOPROVOD SET %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ? WHERE %s = ?",
                Prijmeni.colName(),
                Jmeno.colName(),
                Mejl.colName(),
                Ulice.colName(),
                Mesto.colName(),
                Psc.colName(),
                Telefon.colName(),
                DoprovodId.colName()));
        deletePs = connection.prepareStatement(
                String.format("DELETE FROM DOPROVOD WHERE %s = ?", DoprovodId.colName()));
    }

    @Override
    public void add(Prijmeni prijmeni, Jmeno jmeno, Datum narozeni, Mejl mejl, Ulice ulice, Mesto mesto, Psc psc, Telefon telefon) throws SmpException {
        try {
            int i = 0;
            createPs.setString(++i, prijmeni.getPrijmeni());
            createPs.setString(++i, jmeno.getJmeno());
            createPs.setDate(++i, narozeni.getDate());
            createPs.setString(++i, mejl.getMejl());
            createPs.setString(++i, ulice.getUlice());
            createPs.setString(++i, mesto.getMesto());
            createPs.setString(++i, psc.getPsc());
            createPs.setString(++i, telefon.getTelefon());
            createPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private Doprovod loadDoprovod(ResultSet rs) throws SQLException {
        Doprovod t = new Doprovod(new DoprovodId(rs.getInt(DoprovodId.colName())),
                new Prijmeni(rs.getString(Prijmeni.colName())),
                new Jmeno(rs.getString(Jmeno.colName())),
                new Datum(rs.getDate(OsobaAdresa.COLUMN_NAROZENI)),
                new Mejl(rs.getString(Mejl.colName())),
                new Ulice(rs.getString(Ulice.colName())),
                new Mesto(rs.getString(Mesto.colName())),
                new Psc(rs.getString(Psc.colName())),
                new Telefon(rs.getString(Telefon.colName())));
        return t;
    }

    @Override
    public Doprovod find(DoprovodId doprovodId) throws SmpException {
        try {
            findPs.setInt(1, doprovodId.getAbstrtId());
            ResultSet rs = findPs.executeQuery();
            if (rs.next()) {
                return loadDoprovod(rs);
            }
            return null;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Doprovod> all() throws SmpException {
        try {
            List<Doprovod> list = new ArrayList<>();
            ResultSet rs = findAllPs.executeQuery();
            while (rs.next()) {
                list.add(loadDoprovod(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(DoprovodId doprovodId, Prijmeni prijmeni, Jmeno jmeno, Mejl mejl, Ulice ulice, Mesto mesto, Psc psc, Telefon telefon) throws SmpException {
        try {
            int i = 0;
            updatePs.setString(++i, prijmeni.getPrijmeni());
            updatePs.setString(++i, jmeno.getJmeno());
            updatePs.setString(++i, mejl.getMejl());
            updatePs.setString(++i, ulice.getUlice());
            updatePs.setString(++i, mesto.getMesto());
            updatePs.setString(++i, psc.getPsc());
            updatePs.setString(++i, telefon.getTelefon());
            updatePs.setInt(++i, doprovodId.getAbstrtId());
            updatePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(DoprovodId doprovodId) throws SmpException {
        try {
            deletePs.setInt(1, doprovodId.getAbstrtId());
            deletePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }
}
