package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLIntegrityConstraintViolationException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import org.smp.SmpException;
import pobyty.business.SkupinaDAO;
import pobyty.model.PobytyIntegrityException;
import pobyty.model.doprovod.DoprovodId;
import pobyty.model.skupina.Skupina;
import pobyty.model.skupina.SkupinaId;
import pobyty.model.skupina.VekKlientu;
import pobyty.model.turnus.TurnusId;

public class SkupinaDAODerby implements SkupinaDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement createDoprovodPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updateDoprovodPs;

    public SkupinaDAODerby(Connection connection) throws SQLException {
        createDoprovodPs = connection.prepareStatement("INSERT INTO SKUPINA VALUES(DEFAULT, ?, ?, ?)");
        createPs = connection.prepareStatement("INSERT INTO SKUPINA VALUES(DEFAULT, ?, ?, DEFAULT)");
        findPs = connection.prepareStatement(
                String.format("SELECT * FROM SKUPINA WHERE %s = ?", SkupinaId.colName()));
        findAllPs = connection.prepareStatement(
                String.format("SELECT * FROM SKUPINA WHERE %s = ?", TurnusId.colName()));

        updateDoprovodPs = connection.prepareStatement(
                String.format("UPDATE SKUPINA SET %s = ? WHERE %s = ?",
                DoprovodId.colName(),
                SkupinaId.colName()));

        deletePs = connection.prepareStatement(
                String.format("DELETE FROM SKUPINA WHERE %s = ?", SkupinaId.colName()));
    }

    @Override
    public void add(TurnusId turnusId, VekKlientu vekKlientu, DoprovodId doprovodId) throws SmpException {
        try {
            int i = 0;
            createDoprovodPs.setInt(++i, turnusId.getAbstrtId());
            createDoprovodPs.setInt(++i, vekKlientu.getVek());
            if (doprovodId == null) {
                createDoprovodPs.setNull(++i, Types.INTEGER);
            } else {
                createDoprovodPs.setInt(++i, doprovodId.getAbstrtId());
            }
            createDoprovodPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private static Skupina loadSkupina(ResultSet rs) throws SQLException {
        Skupina s = new Skupina(
                new SkupinaId(rs.getInt(SkupinaId.colName())),
                new TurnusId(rs.getInt(TurnusId.colName())),
                new VekKlientu(rs.getInt(VekKlientu.colName())),
                new DoprovodId(rs.getInt(DoprovodId.colName())));
        return s;
    }

    @Override
    public Skupina find(SkupinaId skupinaId) throws SmpException {
        try {
            findPs.setInt(1, skupinaId.getAbstrtId());
            ResultSet rs = findPs.executeQuery();
            if (!rs.next()) {
                throw new SmpException("Skupina neexistuje: " + skupinaId);
            }
            return loadSkupina(rs);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Skupina> all(TurnusId turnusId) throws SmpException {
        try {
            findAllPs.setInt(1, turnusId.getAbstrtId());
            List<Skupina> list = new ArrayList<>();
            ResultSet rs = findAllPs.executeQuery();
            while (rs.next()) {
                list.add(loadSkupina(rs));
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(SkupinaId skupinaId) throws SmpException, PobytyIntegrityException {
        try {
            deletePs.setInt(1, skupinaId.getAbstrtId());
            deletePs.execute();
        } catch (SQLIntegrityConstraintViolationException ex) {
            throw new PobytyIntegrityException(ex);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(SkupinaId skupinaId, DoprovodId doprovodId) throws SmpException {
        try {
            if (doprovodId == null) {
                updateDoprovodPs.setNull(1, Types.INTEGER);
            } else {
                updateDoprovodPs.setInt(1, doprovodId.getAbstrtId());
            }
            updateDoprovodPs.setInt(2, skupinaId.getAbstrtId());
            updateDoprovodPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }
}
