package pobyty.integration;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLIntegrityConstraintViolationException;
import java.util.ArrayList;
import java.util.List;
import org.smp.SmpException;
import pobyty.business.TurnusDAO;
import pobyty.model.Datum;
import pobyty.model.PobytyIntegrityException;
import pobyty.model.turnus.Cena;
import pobyty.model.turnus.Kapacita;
import pobyty.model.turnus.Lokalita;
import pobyty.model.turnus.Turnus;
import pobyty.model.turnus.TurnusId;
import pobyty.model.turnus.Zkratka;

public class TurnusDAODerby implements TurnusDAO {

    private final PreparedStatement createPs;
    private final PreparedStatement findAllPs;
    private final PreparedStatement deletePs;
    private final PreparedStatement findPs;
    private final PreparedStatement updatePs;

    public TurnusDAODerby(Connection connection) throws SQLException {
        createPs = connection.prepareStatement("INSERT INTO TURNUS VALUES(DEFAULT, ?, ?, ?, ?, ?, ?)");
        findPs = connection.prepareStatement(
                String.format("SELECT * FROM TURNUS WHERE %s = ?", TurnusId.colName()));
        findAllPs = connection.prepareStatement("SELECT * FROM TURNUS");
        updatePs = connection.prepareStatement(
                String.format("UPDATE TURNUS SET %s = ?, %s = ?, %s = ?, %s = ?, %s = ?, %s = ? WHERE %s = ?",
                Zkratka.colName(), Turnus.COLUMN_ZACATEK, Turnus.COLUMN_KONEC, Lokalita.colName(), Cena.colName(), Kapacita.colName(), TurnusId.colName()));
        deletePs = connection.prepareStatement(
                String.format("DELETE FROM TURNUS WHERE %s = ?", TurnusId.colName()));
    }

    @Override
    public void add(Datum zacatek, Datum konec, Lokalita lokalita, Cena cena, Kapacita kapacita, Zkratka zkratka) throws SmpException {
        try {
            int i = 0;
            createPs.setDate(++i, zacatek.getDate());
            createPs.setDate(++i, konec.getDate());
            createPs.setString(++i, lokalita.toString());
            createPs.setInt(++i, cena.getCena());
            createPs.setInt(++i, kapacita.getKapacita());
            createPs.setString(++i, zkratka.getZkratka());
            createPs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    private static Turnus loadTurnus(ResultSet rs) throws SQLException {
        return new Turnus(
                new TurnusId(rs.getInt(TurnusId.colName())),
                new Datum(rs.getDate(Turnus.COLUMN_ZACATEK)),
                new Datum(rs.getDate(Turnus.COLUMN_KONEC)),
                new Lokalita(rs.getString(Lokalita.colName())),
                new Cena(rs.getInt(Cena.colName())),
                new Kapacita(rs.getInt(Kapacita.colName())),
                new Zkratka(rs.getString(Zkratka.colName())));
    }

    @Override
    public Turnus find(TurnusId turnusId) throws SmpException {
        try {
            findPs.setInt(1, turnusId.getAbstrtId());
            ResultSet rs = findPs.executeQuery();
            if (!rs.next()) {
                throw new SmpException("Turnus neexistuje.");
            }
            return loadTurnus(rs);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public List<Turnus> all() throws SmpException {
        try {
            List<Turnus> list = new ArrayList<>();
            ResultSet rs = findAllPs.executeQuery();
            while (rs.next()) {
                Turnus t = loadTurnus(rs);
                list.add(t);
            }
            return list;
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void update(TurnusId turnusId, Zkratka zkratka, Datum zacatek, Datum konec, Lokalita lokalita, Cena cena, Kapacita kapacita) throws SmpException {
        try {
            int i = 0;
            updatePs.setString(++i, zkratka.getZkratka());
            updatePs.setDate(++i, zacatek.getDate());
            updatePs.setDate(++i, konec.getDate());
            updatePs.setString(++i, lokalita.toString());
            updatePs.setInt(++i, cena.getCena());
            updatePs.setInt(++i, kapacita.getKapacita());
            updatePs.setInt(++i, turnusId.getAbstrtId());
            updatePs.execute();
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }

    @Override
    public void delete(TurnusId turnusId) throws SmpException, PobytyIntegrityException {
        try {
            deletePs.setInt(1, turnusId.getAbstrtId());
            deletePs.execute();
        } catch (SQLIntegrityConstraintViolationException ex) {
            throw new PobytyIntegrityException(ex);
        } catch (SQLException ex) {
            throw new SmpException(ex);
        }
    }
}
