package pobyty.model.skupina;

import java.util.logging.Level;
import java.util.logging.Logger;
import org.smp.SmpException;
import pobyty.business.Facade;
import pobyty.model.doprovod.Doprovod;
import pobyty.model.doprovod.DoprovodId;
import pobyty.model.turnus.Turnus;
import pobyty.model.turnus.TurnusId;

public final class Skupina {

    public static Skupina NEDEF = new Skupina();
    private SkupinaId skupinaId;
    private TurnusId turnusId;
    private Turnus turnus;
    private VekKlientu vekKlientu;
    private DoprovodId doprovodId;

    public Skupina(SkupinaId skupinaId, TurnusId turnusId, VekKlientu vekKlientu, DoprovodId doprovodId) {
        this.skupinaId = skupinaId;
        this.turnusId = turnusId;
        this.vekKlientu = vekKlientu;
        this.doprovodId = doprovodId;
    }

    private Skupina() {
    }

    /**
     * @return the getSkupinaId
     */
    public SkupinaId getSkupinaId() {
        return skupinaId;
    }

    public VekKlientu getVekKlientu() {
        return vekKlientu;
    }

    public Turnus getTurnus() throws SmpException {
        if (turnus == null) {
            turnus = Facade.getInstance().findTurnus(getTurnusId());
        }
        assert turnus.getTurnusId().equals(getTurnusId());
        return turnus;
    }

    public Integer getPocetKlientu() throws SmpException {
        return Facade.getInstance().pocetKlientuSkupiny(skupinaId);
    }

    @Override
    public String toString() {
        try {
            if (this == NEDEF) {
                return "neurčená";
            }
            return String.format("%2d:%s", skupinaId.getAbstrtId(), getTurnus().getZkratka());
        } catch (SmpException ex) {
            Logger.getLogger(Skupina.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    /**
     * @return the getDoprovodId
     */
    public DoprovodId getDoprovodId() {
        return doprovodId;
    }

    /**
     * @param getDoprovodId the getDoprovodId to set
     */
    public void setDoprovodId(DoprovodId doprovodId) {
        this.doprovodId = doprovodId;
    }

    /**
     * @return the doprovod
     */
    public Doprovod getDoprovod() throws SmpException {
        if (DoprovodId.NEDEF.equals(doprovodId)) {
            return Doprovod.NEDEF;
        }
        return Facade.getInstance().findDoprovod(doprovodId);
    }

    /**
     * @return the turnusId
     */
    public TurnusId getTurnusId() {
        return turnusId;
    }
}
