package pobyty.model.turnus;

import org.smp.SmpException;
import pobyty.business.Facade;
import pobyty.model.Datum;

public class Turnus implements Comparable<Turnus> {

    public static final String COLUMN_ZACATEK = "Zacatek";
    public static final String COLUMN_KONEC = "Konec";
    private TurnusId turnusId;
    private Datum zacatek;
    private Datum konec;
    private Lokalita lokalita;
    private Cena cena;
    private Kapacita kapacita;
    private Zkratka zkratka;

    public Turnus() {
    }

    public Turnus(TurnusId turnusId, Datum zacatek, Datum konec, Lokalita lokalita, Cena cena, Kapacita kapacita, Zkratka zkratka) {
        this.turnusId = turnusId;
        this.zacatek = zacatek;
        this.konec = konec;
        this.lokalita = lokalita;
        this.cena = cena;
        this.kapacita = kapacita;
        this.zkratka = zkratka;
    }
    public static int TURNUS_SIZE = 15;

    @Override
    public String toString() {
        return String.format("%s  %s", getZacatek(), lokalita);
    }

    /**
     * @return the turnusId
     */
    public TurnusId getTurnusId() {
        return turnusId;
    }

    /**
     * @return the zacatek
     */
    public Datum getZacatek() {
        return zacatek;
    }

    /**
     * @return the konec
     */
    public Datum getKonec() {
        return konec;
    }

    /**
     * @return the lokalita
     */
    public Lokalita getLokalita() {
        return lokalita;
    }

    /**
     * @return the cena
     */
    public Cena getCena() {
        return cena;
    }

    /**
     * Get the value of kapacita
     *
     * @return the value of kapacita
     */
    public Kapacita getKapacita() {
        return kapacita;
    }

    public int getVolnaKapacita() throws SmpException {
        return getKapacita().getKapacita() - getPrihlasenych();
    }

    public int getPrihlasenych() throws SmpException {
        return Facade.getInstance().pocetPrihlasenychNaTurnus(turnusId);
    }

    public Zkratka getZkratka() {
        return zkratka;
    }

    public void setZacatek(Datum zacatek) {
        this.zacatek = zacatek;
    }

    /**
     * @param konec the konec to set
     */
    public void setKonec(Datum konec) {
        this.konec = konec;
    }

    /**
     * @param lokalita the lokalita to set
     */
    public void setLokalita(Lokalita lokalita) {
        this.lokalita = lokalita;
    }

    /**
     * @param cena the cena to set
     */
    public void setCena(Cena cena) {
        this.cena = cena;
    }

    /**
     * @param kapacita the kapacita to set
     */
    public void setKapacita(Kapacita kapacita) {
        this.kapacita = kapacita;
    }

    /**
     * @param zkratka the zkratka to set
     */
    public void setZkratka(Zkratka zkratka) {
        this.zkratka = zkratka;
    }

    @Override
    public int compareTo(Turnus that) {
        int compId = turnusId.compareTo(that.getTurnusId());
        if (compId == 0) {
            return 0;
        }
        int compZac = getZacatek().compareTo(that.getZacatek());
        if (compZac == 0) {
            return compId;
        }
        return compZac;
    }

    @Override
    public boolean equals(Object that) {
        if (that == null) {
            return false;
        }
        assert that.getClass() == Turnus.class;
        return compareTo((Turnus) that) == 0;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 67 * hash + turnusId.hashCode();
        return hash;
    }
}
